/****************************************************************************/
/*  Copyright 2005, Trustees of Indiana University                          */
/*  Author: Sidharth Thakur                                                 */
/*  Last modified: 14 January 2006                                          */
/*  License available at: <belt-trick-* folder>/docs/downloads/License.txt  */
/****************************************************************************/
#ifndef _QUATSET_H_
#define _QUATSET_H_

///////////////////////////////////////////////////////////////////////////////
//
// $Header:  MathObject.h$
//
// NAME: QuaternionSet
//
// BASE  CLASSES: none
//  
// PURPOSE: Class header for describing QuaternionSet object
//
// AUTHOR: Sidharth Thakur
//
// DATE: Wed Feb 23 12:10:15 EST 2005
//
// COPYRIGHT NOTICE: "Copyright 2005, Trustees of Indiana University, Hanson 
// Visualization Laboratory"
//
//  REVISION HISTORY:
// $Log:      $
///////////////////////////////////////////////////////////////////////////////

#include "all.h"

class QuaternionSet {

	public:
	QuaternionSet();
	~QuaternionSet();

	int				id;

	//////////////////////////////////////////////////////////////////////////
	/* DATA STRUCTURES */
	quaternion	**quatPointSet;
	quaternion  **quatPointSetNew;
	Vertex		*pset;					// set of points on curve
	Vertex		*tset;					// set of tangents on curve
	Vertex		*tbset;
	int			numqset;				// # of quaternion points
	int			numpset;				// # of curve points
	int			numtset;				// # of tangent points
	int			whichQuatComponent;
	int			showQuatPoints;
	int			showQuatCurve;
	int			quatkeyframe;

	//////////////////////////////////////////////////////////////////////////
	/* DRAWING ROUTINES */										
	//////////////////////////////////////////////////////////////////////////
	void		displayQuaternions		( void );
	void		renderThickenedCurve	( double radius );

	//////////////////////////////////////////////////////////////////////////
	/* ADMIN ROUTINES */										
	//////////////////////////////////////////////////////////////////////////
 public:
 void  initObject    ( void );       // 
 void  initStructs    ( void );       //
 void  printStatus    ( void );       //
 void  setPointSet    ( Vertex *refPointSet, int numPoints );  //
 void  setTangSet    ( Vertex *refTangSet, int numPoints );  //
 void  setTubePointSet   ( Vertex *pointList, int numPoints );  //
 void  printVec    ( quaternion* q); 

 //////////////////////////////////////////////////////////////////////////////////////////////////
 /* COMPUTATION ROUTINES */          
 //////////////////////////////////////////////////////////////////////////////////////////////////
 public:
 void  computeQuaternionSet ( int mode );
 void  generateQuaternionRings ( void );
 void  makeFamilyOfQuaternions ( double theta, double *vector, quaternion *q );   //
 void  optimize    ( void );
 void  optimize2    ( void );
 void  curveQuatLength   ( void );

};

#endif
