/****************************************************************************/
/*  Copyright 2005, Trustees of Indiana University                          */
/*  Author: Sidharth Thakur                                                 */
/*  Last modified: 14 January 2006                                          */
/*  License available at: <belt-trick-* folder>/docs/downloads/License.txt  */
/****************************************************************************/
#include "objectBelt.h"
#include "QuaternionSet.h"

//global declarations
//QuaternionSet	*qset;
MathObject	*mo1;
int			whichBelt;
int			numBeltPoints;
double		angle;
double		factor;
double		displacement_edge;
int			sldrBeltMaxValue;
Vertex		*edges;
static GLUquadricObj *qobjCYL, *qobjSPH;

/* Material structures */
static const GLfloat no_mat[]					= {0.0f,0.0f,0.0f,1.0f};
static const GLfloat mat_ambient[]				= {0.2f,0.2f,0.6f,1.0f};
static const GLfloat mat_ambient_color[]		= {0.5f,0.5f,0.5f,1.0f};
static const GLfloat mat_diffuse[]				= {0.1f,0.35f,0.65f,1.0f};
static const GLfloat mat_specular[]				= {1.0f,1.0f,1.0f,1.0f};
static const GLfloat no_shininess[]				= {0.0f};
static const GLfloat low_shininess[]			= {5.0f};
static const GLfloat high_shininess[]			= {100.0f};
static const GLfloat mat_emission[]				= {0.3f,0.2f,0.2f,0.0f};

static const GLfloat no_mat2[]					= {0.0f,0.0f,0.0f,1.0f};
static const GLfloat mat_ambient2[]				= {0.6f,0.1f,0.3f,1.0f}; 
static const GLfloat mat_ambient_color2[]		= {0.5f,0.5f,0.5f,1.0f};
static const GLfloat mat_diffuse2[]				= {0.65f,0.1f,0.35f,1.0f};
static const GLfloat mat_specular2[]			= {1.0f,1.0f,1.0f,1.0f};
static const GLfloat no_shininess2[]			= {0.0f};
static const GLfloat low_shininess2[]			= {5.0f};
static const GLfloat high_shininess2[]			= {100.0f};
static const GLfloat mat_emission2[]			= {0.3f,0.2f,0.2f,0.0f};

static const GLfloat no_mat3[]					= {0.0f,0.0f,0.0f,1.0f};
static const GLfloat mat_ambient3[]				= {0.6f,0.4f,0.0f,1.0f}; 
static const GLfloat mat_ambient_color3[]		= {0.5f,0.5f,0.5f,1.0f};
static const GLfloat mat_diffuse3[]				= {0.85f,0.45f,0.1f,1.0f};
static const GLfloat mat_specular3[]			= {1.0f,1.0f,1.0f,1.0f};
static const GLfloat no_shininess3[]			= {0.0f};
static const GLfloat low_shininess3[]			= {5.0f};
static const GLfloat high_shininess3[]			= {100.0f};
static const GLfloat mat_emission3[]			= {0.3f,0.2f,0.2f,0.0f};

static const GLfloat no_mat4[]					= {0.0f,0.0f,0.0f,1.0f};
static const GLfloat mat_ambient4[]				= {0.3f,0.5f,0.0f,1.0f}; 
static const GLfloat mat_ambient_color4[]		= {0.5f,0.5f,0.5f,1.0f};
static const GLfloat mat_diffuse4[]				= {0.35f,0.75f,0.1f,1.0f};
static const GLfloat mat_specular4[]			= {1.0f,1.0f,1.0f,1.0f};
static const GLfloat no_shininess4[]			= {0.0f};
static const GLfloat low_shininess4[]			= {5.0f};
static const GLfloat high_shininess4[]			= {100.0f};
static const GLfloat mat_emission4[]			= {0.3f,0.2f,0.2f,0.0f};


char obj_qstr[128];

void text(char* string) {
  char* p;
  for(p=string;*p;p++) {
    glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18,*p);
  }
}

void showOnScreenText( int num ) 
{
	double xpos=0.0;
	double ypos=0.0;
    
    glPushAttrib(GL_CURRENT_BIT | GL_LIGHTING_BIT);
    glDisable(GL_LIGHTING);
    
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
        glLoadIdentity();
        glMatrixMode(GL_PROJECTION);
        glPushMatrix();
        glLoadIdentity();
        gluOrtho2D(-1.2,1.2,-1.2,4.2);

        glMatrixMode(GL_MODELVIEW);
        glDisable(GL_LIGHTING);

        xpos=0.0;
        ypos=0.0;

		glColor3f(0.0f,0.0f,0.7f);
		sprintf(obj_qstr,"%d",num);
		glRasterPos2f(xpos,ypos);
		text(obj_qstr);

		glMatrixMode(GL_PROJECTION);   
        glPopMatrix();
        glMatrixMode(GL_MODELVIEW);
    glPopMatrix();
    glPopAttrib();
}
/********************************************************************************
* Date   : Wed Jul 20 13:13:56 EST 2005
* Method : void cleanup ( void )
* Args   : none
* Return : none
* Purpose: Cleanup on application Exit or Reset
********************************************************************************/
void cleanup ( void ) {
	my_free(mo1->bquatlist);
	my_free(mo1->bpntlist);
	my_free(mo1->tpntlist);
	my_free(mo1->npntlist);
}

/********************************************************************************
* Date   : Wed Jul 20 13:13:56 EST 2005
* Method : void initBeltFirst ( MathObject* mathobjRef )
* Args   : pointer to object of Class MathObject
* Return : none
* Purpose: called on application Load or Reset
********************************************************************************/
void initBeltFirst ( MathObject* mathobjRef ) {
	
	int i			= 0;
	int numpoints	= 0;
	double angles[3],color[3];

	mo1 = mathobjRef;
    mo1->bspheres		= 200;
	numpoints			= mo1->bspheres;

	//# Belt data structures
	mo1->bquatlist = ( quaternion* ) my_malloc( sizeof(quaternion)*numpoints);
	for(i=0;i<numpoints;i++) {
        mo1->bquatlist[i].r = 0.0;
        mo1->bquatlist[i].i = 0.0;
        mo1->bquatlist[i].j = 0.0;
        mo1->bquatlist[i].k = 0.0;
	}
    
	edges = (Vertex*) my_malloc(sizeof(Vertex)*2);
	for(i=0;i<2;i++) {
        edges[i].p3.d[0] = 0.0;
        edges[i].p3.d[1] = 0.0;
        edges[i].p3.d[2] = 0.0;
	}

	//# Belt parameters
	mo1->balpha			= 0.0;
    mo1->bsetupalpha	= 0.0;
    mo1->bmultiple		= 1;
	mo1->bhiderest		= 0;
	mo1->bpattern		= BELT_PATTERN_DOUBLE_SIDED;
    mo1->bspheresFactor = 30;
	mo1->bspine			= 1;
	mo1->btrackerpos	= 0;
	mo1->bwidth			= 0.0;
	mo1->bpntlist		= (Vertex*) my_malloc(sizeof(Vertex)*numpoints);
	for(i=0;i<numpoints;i++) {
        mo1->bpntlist[i].p3.d[0] = 0.0;
        mo1->bpntlist[i].p3.d[1] = 0.0;
        mo1->bpntlist[i].p3.d[2] = 0.0;
	}
	mo1->tpntlist		= (Vertex*) my_malloc(sizeof(Vertex)*numpoints);
	for(i=0;i<numpoints;i++) {
        mo1->tpntlist[i].p3.d[0] = 0.0;
        mo1->tpntlist[i].p3.d[1] = 0.0;
        mo1->tpntlist[i].p3.d[2] = 0.0;
	}	
	mo1->npntlist		= (Vertex*) my_malloc(sizeof(Vertex)*numpoints);
	for(i=0;i<numpoints;i++) {
        mo1->npntlist[i].p3.d[0] = 0.0;
        mo1->npntlist[i].p3.d[1] = 0.0;
        mo1->npntlist[i].p3.d[2] = 0.0;
	}	
	mo1->br				= (Vertex*) my_malloc(sizeof(Vertex)*numpoints);
	for(i=0;i<numpoints;i++) {
        mo1->br[i].p3.d[0] = 0.0;
        mo1->br[i].p3.d[1] = 0.0;
        mo1->br[i].p3.d[2] = 0.0;
	}	
	mo1->bl				= (Vertex*) my_malloc(sizeof(Vertex)*numpoints);
	for(i=0;i<numpoints;i++) {
        mo1->bl[i].p3.d[0] = 0.0;
        mo1->bl[i].p3.d[1] = 0.0;
        mo1->bl[i].p3.d[2] = 0.0;
	}	
	mo1->qset			= new QuaternionSet();
	mo1->qset->whichQuatComponent		= SHOWWYZ;
	mo1->inp_chkbox_belt_show_numbers	= 0;
	mo1->inp_chkbox_belt_spheres		= 0;
    mo1->inp_chkbox_belt_spheres_wire	= 0;
    mo1->inp_chkbox_belt_type			= BELT360;
	mo1->inp_slider_belt_angle			= 0.0;
    mo1->inp_slider_belt_setup_angle	= 0.0;
	mo1->inp_slider_belt_width			= 1.0;


	//# View parameters
    mo1->showAxes       = 1;
	mo1->showFrame		= 0;
	mo1->showFace		= 1;
	mo1->showEdgeOnly   = 0;
    mo1->showEdge       = 1;
	mo1->showSphere		= 0;
	mo1->showTracker	= 1;

	//# Quaternion map parameters
	mo1->qshowAxes		= 1;

	//# Global parameters
    factor				= 0.0;
	displacement_edge	= 0.001;
	sldrBeltMaxValue	= 100;
	qobjCYL				= gluNewQuadric();
	qobjSPH				= gluNewQuadric();

	genBeltAngles(0,angles,color);
	initBelt(angles[0]);

	//# Global Flags
	mo1->flag_blend		= 1;
	mo1->flag_track_belt= -1;
	mo1->flag_do_not_compute = 0;
}

/********************************************************************************
* Date   : Wed Jul 20 13:13:56 EST 2005
* Method : void initBelt ( void )
* Args   : none
* Return : none
* Purpose: initialize belt-state in each rendering loop
********************************************************************************/
void initBelt ( double quat_angle )
{
    int		i		= 0;
    double	delta	= 0.0;
    double	alpha	= 0.0;
    double	t		= 0.0;
	double	gamma	= 0.0;
	double	l_qangle= quat_angle;
	l_qangle = 0.0;

    numBeltPoints	= mo1->bspheres;
	delta			= 1.0/(double)numBeltPoints;

    quaternion* q;
    q = (quaternion*) my_malloc(sizeof(quaternion));
    
    if(mo1->inp_chkbox_belt_type == BELT360) {
        factor = 1.0*M_PI/(double)sldrBeltMaxValue;
    }
    else if(mo1->inp_chkbox_belt_type == BELT720) {
        factor = 0.5*M_PI/(double)sldrBeltMaxValue;
    }
	
	alpha = ((double)mo1->inp_slider_belt_angle)*factor;
	
	mo1->balpha = alpha;

    if(mo1->inp_chkbox_belt_type == BELT360) {
        gamma = ((double)mo1->inp_slider_belt_setup_angle)*factor;
    }
    else if(mo1->inp_chkbox_belt_type == BELT720) {
        gamma = 4.0*((double)mo1->inp_slider_belt_setup_angle)*factor;
    }	
	mo1->bsetupalpha = gamma;

	whichBelt = mo1->inp_chkbox_belt_type; 

	switch(whichBelt) {

		case BELT360:
			t = 0.0;
			for( i=0; i<numBeltPoints; i++) {
				computeBelt360( alpha, gamma, t, q );	
				mo1->bquatlist[i].r = q->r;
				mo1->bquatlist[i].i = q->i;
				mo1->bquatlist[i].j = q->j;
				mo1->bquatlist[i].k = q->k;
				//computeBelt360( alpha+quat_angle, gamma, t, q );
				mo1->qset->quatPointSetNew[i]->r = q->r;
				mo1->qset->quatPointSetNew[i]->i = q->i;
				mo1->qset->quatPointSetNew[i]->j = q->j;
				mo1->qset->quatPointSetNew[i]->k = q->k;

				t = t+delta;
			}
			mo1->qset->numqset = numBeltPoints;

			break;
		
		case BELT720:		
			t = 0.0;
			for(i=0;i<numBeltPoints;i++) {
				computeBelt720( alpha, gamma, t, q );	
				mo1->bquatlist[i].r = q->r;
				mo1->bquatlist[i].i = q->i;
				mo1->bquatlist[i].j = q->j;
				mo1->bquatlist[i].k = q->k;
				//computeBelt720( alpha+quat_angle, gamma, t, q );
				mo1->qset->quatPointSetNew[i]->r = q->r;
				mo1->qset->quatPointSetNew[i]->i = q->i;
				mo1->qset->quatPointSetNew[i]->j = q->j;
				mo1->qset->quatPointSetNew[i]->k = q->k;
				
				t = t+delta;
			}
			mo1->qset->numqset = numBeltPoints;
                        break;

		default:
			break;
	}
	my_free(q);
}

/********************************************************************************
* Date   : Wed Jul 20 13:13:56 EST 2005
* Method : void displayBelt ( void )
* Args   : none
* Return : none
* Purpose: draw belt in each rendering loop
********************************************************************************/
void displayBelt ( void ) {
	glPushMatrix();
		glRotatef(15.0,1.0,0.0,0.0);
		glRotatef(-15.0,0.0,1.0,0.0);
		glScalef(0.9f,0.9f,0.9f);

		if(mo1->showAxes)
			renderAxes();
		initBelt(M_PI/2.0);
		constructBelt();
	glPopMatrix();
}

/********************************************************************************
* Date   : Wed Jul 20 13:13:56 EST 2005
* Method : void displayBeltQuaternionMap ( void )
* Args   : none
* Return : none
* Purpose: draw Quaternion Map in each rendering loop
********************************************************************************/
void displayBeltQuaternionMap ( void )
{
	glPushMatrix();

	if(mo1->qshowAxes)
		renderAxes();

	glScalef(1.5f,1.5f,1.5f);
	mo1->qset->showQuatPoints	= 1;
	mo1->qset->showQuatCurve	= 1;
	mo1->qset->quatkeyframe		= mo1->btrackerpos;
	mo1->qset->displayQuaternions();
	
	glPopMatrix();
}

/********************************************************************************
* Date   : Wed Jul 20 13:13:56 EST 2005
* Method : void computeBelt0 ( void )
* Args   : none
* Return : none
* Purpose: compute the belt-state during the initial setup stage
********************************************************************************/
void computeBelt0 ( double alpha, double t, quaternion* q )
{
	q->r = 0.0;
	q->i = cos(t*alpha);
	q->j = sin(t*alpha);
	q->k = 0.0;
}

/********************************************************************************
* Date   : Wed Jul 20 13:13:56 EST 2005
* Method : void computeBelt360 ( void )
* Args   : none
* Return : none
* Purpose: compute belt-state during belt-transformation for 360-degree trick
********************************************************************************/
void computeBelt360 ( double alpha, double gamma, double t, quaternion* q )
{
	if(gamma<=(M_PI+0.06)) {
		q->r = cos(t*gamma);
		q->i = 0.0;
		q->j = sin(alpha)*sin(t*gamma);
		q->k = cos(alpha)*sin(t*gamma);
	}
	else {
		q->r = cos(t*M_PI);
		q->i = 0.0;
		q->j = sin(alpha)*sin(t*M_PI);
		q->k = cos(alpha)*sin(t*M_PI);
	}
		//q->r = cos(t*gamma)*cos(t*M_PI);
		//q->i = 0.0;
		//q->j = sin(t*gamma)*sin(alpha)*sin(t*M_PI);
		//q->k = cos(t*gamma)*cos(alpha)*sin(t*M_PI);

}

/********************************************************************************
* Date   : Wed Jul 20 13:13:56 EST 2005
* Method : void computeBelt720 ( void )
* Args   : none
* Return : none
* Purpose: compute belt-state during belt-transformation for 720-degree trick
********************************************************************************/
void computeBelt720 ( double beta, double gamma, double t, quaternion* q )
{
	if(gamma<=(2.0*M_PI)) {
		q->r = cos(t*gamma);
		q->i = 0.0;
		q->j = sin(beta)*sin(t*gamma);
		q->k = cos(beta)*sin(t*gamma);
	}
	else {
		q->r = sin(beta)*sin(beta) + cos(beta)*cos(beta)*cos(2.0*t*M_PI);
		q->i = 0;
		q->j = cos(beta)*sin(beta)*(1 - cos(2.0*t*M_PI));
		q->k = cos(beta)*sin(2.0*t*M_PI);
	}
}

/********************************************************************************
* Date   : Wed Jul 20 13:13:56 EST 2005
* Method : void renderAxes ( void )
* Args   : none
* Return : none
* Purpose: draw world-axes during each loop in rendering
********************************************************************************/
void renderAxes ( void )
{
    double len = 0.5;
    double rad = 0.025;
    
    GLUquadricObj *qobj;
    qobj = gluNewQuadric();

    glPushAttrib(GL_CURRENT_BIT | GL_LIGHTING_BIT);
       glPushMatrix();

          glDisable(GL_LIGHTING);
          glPushMatrix();
             glTranslatef(0.0,0.0,0.0);
             glColor3f(0.0, 0.0, 1.0);
             glRotatef(-90,1,0,0);
             gluQuadricDrawStyle(qobj, GLU_FILL);
             gluCylinder(qobj, rad, rad, len, 16, 16);
          glPopMatrix();
          glPushMatrix();
             glTranslatef(0.0,0.0,0.0);
             glColor3f(0.0, 1.0, 0.0);
             glRotatef(90,0,1,0);
             gluQuadricDrawStyle(qobj, GLU_FILL);
             gluCylinder(qobj, rad, rad, len, 16, 16);
          glPopMatrix();
          glPushMatrix();
             glTranslatef(0.0,0.0,0.0);
             glColor3f(1.0, 0.0, 0.0);
             glRotatef(0,0,0,1);
             gluQuadricDrawStyle(qobj, GLU_FILL);
             gluCylinder(qobj, rad, rad, len, 16, 16);
          glPopMatrix();

       glPopMatrix();
    glPopAttrib();
}

/********************************************************************************
* Date   : Wed Jul 20 13:13:56 EST 2005
* Method : void genSpherePoint( double radius, double phi, double theta, double* point )
* Args   : radius of bounding sphere, parameter angles - phi and theta, array of points
* Return : none
* Purpose: compute and return points on the bounding sphere
********************************************************************************/
void genSpherePoint( double radius, double phi, double theta, double* point )
{
	point[0] = radius*cos(phi)*cos(theta);
	point[1] = radius*cos(phi)*sin(theta);
	point[2] = radius*sin(phi);
}

/********************************************************************************
* Date   : Wed Jul 20 13:13:56 EST 2005
* Method : void genBeltAngles ( int i, double *angles, double *color )
* Args   : index, paramter angles for belt, color of belt
* Return : none
* Purpose: returns the angles for computation of belt segments and color of belt
********************************************************************************/
void genBeltAngles ( int i, double *angles, double *color )
{
	switch(i) {
		case 0:
			angles[0] = M_PI/2.0;
			angles[1] = 0.0;
			color[0] = 0.1; color[1] = 0.2; color[2] = 0.85;
			break;
		case 1:
			angles[0] = -M_PI/2.0;
			angles[1] = 0.0;
			color[0] = 0.8; color[1] = 0.2; color[2] = 0.2;
			break;
		case 2:
			angles[0] = M_PI;
			angles[1] = 0.0;
			color[0] = 0.2; color[1] = 0.2; color[2] = 0.8;
			break;
		case 3:
			angles[0] = 2.0*M_PI;
			angles[1] = 0.0;
			color[0] = 0.8; color[1] = 0.2; color[2] = 0.2;
			break;
		case 4:
			angles[0] = 0.0;
			angles[1] = M_PI/2.0;
			color[0] = 0.1; color[1] = 0.3; color[2] = 0.9;
			break;
		case 5:
			angles[0] = 0.0;
			angles[1] = -M_PI/2.0;
			color[0] = 0.9; color[1] = 0.3; color[2] = 0.1;
			break;
		case 6:
			angles[0] = M_PI/4.0;
			angles[1] = -M_PI/2.0;
			color[0] = 0.2; color[1] = 0.8; color[2] = 0.2;
			break;
		case 7:
			angles[0] = M_PI/4.0;
			angles[1] = M_PI/2.0;
			color[0] = 0.2; color[1] = 0.8; color[2] = 0.2;
			break;
		case 8:
			angles[0] = -M_PI/4.0;
			angles[1] = -M_PI/2.0;
			color[0] = 0.2; color[1] = 0.8; color[2] = 0.2;
			break;
		case 9:
			angles[0] = -M_PI/4.0;
			angles[1] = M_PI/2.0;
			color[0] = 0.2; color[1] = 0.8; color[2] = 0.2;
			break;
		case 10:
			angles[0] = M_PI;
			angles[1] = M_PI/4.0;
			color[0] = 0.8; color[1] = 0.2; color[2] = 0.8;
			break;
		case 11: 
			angles[0] = M_PI;
			angles[1] = -M_PI/4.0;
			color[0] = 0.8; color[1] = 0.2; color[2] = 0.8;
			break;
		case 12:
			angles[0] = 0.0;
			angles[1] = M_PI/4.0;
			color[0] = 0.8; color[1] = 0.2; color[2] = 0.8;
			break;
		case 13:
			angles[0] = 0.0;
			angles[1] = -M_PI/4.0;
			color[0] = 0.8; color[1] = 0.2; color[2] = 0.8;
			break;

		default:
			break;
	}
}


/********************************************************************************
* Date   : Wed Jul 20 13:13:56 EST 2005
* Method : void setupBelt ( void )
* Args   : none
* Return : none
* Purpose: compute belt-state during setup-stage
********************************************************************************/
void setupBelt ( void )
{
    int     i			= 0;
	double	delta		= 1.0/(double)numBeltPoints;
    double  beta		= 0.0;
    double  t			= 0.0;
	double	scale		= 1.0;
    double	mat[16];    
	double	v[3], p[3];
    
    quaternion* q;
    q = (quaternion*) my_malloc(sizeof(quaternion));

    if(mo1->inp_chkbox_belt_type == BELT360) {
        factor = 2.0*M_PI/(double)sldrBeltMaxValue;
    }
    else if(mo1->inp_chkbox_belt_type == BELT720) {
        factor = 4.0*M_PI/(double)sldrBeltMaxValue;
    }

	beta = ((double)mo1->inp_slider_belt_setup_angle)*factor;
	//printf("setup angle val = %d\n",mo1->inp_slider_belt_setup_angle);
	for( i=0; i<numBeltPoints; i++ ) {
		
		computeBelt0 ( beta, t, q );

		quatToRotationMatrix( q, mat );

		//mo1->qset->quatPointSetNew[i]->r = q->r;
		//mo1->qset->quatPointSetNew[i]->i = q->i;
		//mo1->qset->quatPointSetNew[i]->j = q->j;
		//mo1->qset->quatPointSetNew[i]->k = q->k;
		
		p[0] = mo1->bpntlist[i].p3.d[0];
		p[1] = mo1->bpntlist[i].p3.d[1];
		p[2] = mo1->bpntlist[i].p3.d[2];

		v[0] = p[0]*mat[0] + p[1]*mat[4] + p[2]*mat[8];
		v[1] = p[0]*mat[1] + p[1]*mat[5] + p[2]*mat[9];
		v[2] = scale*(p[0]*mat[2] + p[1]*mat[6] + p[2]*mat[10]);

		mo1->bpntlist[i].p3.d[0] = v[0];
		mo1->bpntlist[i].p3.d[1] = v[1];
		mo1->bpntlist[i].p3.d[2] = v[2];

		t = t+delta;
	}   
	my_free(q);
}

/********************************************************************************
* Date   : Wed Jul 20 13:13:56 EST 2005
* Method : void constructBelt ( void )
* Args   : none
* Return : none
* Purpose: compute and render belt segments, local frames, bounding spheres, 
*		   edges, etc.
********************************************************************************/
void constructBelt ( void ) 
{
	int		i			= 0;
	int		numBelts	= 0;
	int		beltnum		= 0;
	int		spherenum	= 0;
	double	sphererad	= 0.0;
	double	d			= 2.0/mo1->bspheres;
	double	dd			= 0.0;
	double	a			= 0.0;
	double	scale		= 1.0;
	double	len			= 0.0;
	double	frmscale	= 0.25;
	int		trackerpos	= 0;
	double	mat[16];
	float	matf[16];
	double	v1[3],v2[3],v3[3];
	double	v[3],p[3];

	double	angles[2];
	double	color[3];
	double	p1[3],p2[3];
	double  initFrame[9];

    GLUquadricObj *qobjFrames;
    qobjFrames = gluNewQuadric();

	mo1->bwidth = ((double)mo1->inp_slider_belt_width)/100.0;;
	numBelts = mo1->bmultiple;

	//##########################################################
	for(beltnum = 0; beltnum<numBelts; beltnum++) {

		//genBeltAngles(0,angles,color);
		//initBelt(angles[0]);

		dd = 0.0;
		
		//Belt: compute points on belt
		for(i=0; i<numBeltPoints; i++) {

			quatToRotationMatrix(&(mo1->bquatlist[i]),mat);
			dd = dd+d;
			v1[0] = mat[0]; v1[1] = mat[4]; v1[2] = mat[8];
			v2[0] = mat[1]; v2[1] = mat[5]; v2[2] = mat[9];
			v3[0] = mat[2]; v3[1] = mat[6]; v3[2] = mat[10];

			genBeltAngles(beltnum,angles,color);
			genSpherePoint(dd,angles[0],angles[1],p);

            mo1->tpntlist[i].p3.d[0] = p[0];
			mo1->tpntlist[i].p3.d[1] = p[1];
			mo1->tpntlist[i].p3.d[2] = p[2];

			v[0] = p[0]*mat[0] + p[1]*mat[4] + p[2]*mat[8];
			v[1] = p[0]*mat[1] + p[1]*mat[5] + p[2]*mat[9];
			v[2] = scale*(p[0]*mat[2] + p[1]*mat[6] + p[2]*mat[10]);

			mo1->bpntlist[i].p3.d[0] = v[0];
			mo1->bpntlist[i].p3.d[1] = v[1];
			mo1->bpntlist[i].p3.d[2] = v[2];
			//Points on belts
			/*
				glPushMatrix();
				glTranslatef(v[0],v[1],v[2]);
				glutSolidSphere(0.01,16,16);
				glPopMatrix();
			*/
			glPushAttrib(GL_LIGHTING_BIT | GL_LINE_BIT | GL_POINT_BIT);
			glLineWidth(2.0f);

		}

		//Belt: Belt Spine
		////if(beltnum == 0 && mo1->bspine) {
		////	glPushAttrib(GL_ALL_ATTRIB_BITS);
		////	glEnable(GL_LIGHTING);
		////	glEnable(GL_BLEND);
		////	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		////	glEnable(GL_LINE_SMOOTH);
		////	glColor3f(0.7f, 0.3f, 0.0f);
		////	renderThickenedCurve(numBeltPoints+1, 0.015, mo1->bpntlist);
		////	//glBegin(GL_LINE_STRIP);
		////	//for(i=0; i<numBeltPoints; i++) {
		////	//	glVertex3f(mo1->bpntlist[i].p3.d[0], mo1->bpntlist[i].p3.d[1], mo1->bpntlist[i].p3.d[2]);
		////	//}
		////	//glEnd();
		////	glPopAttrib();
		////}




		//Belt Segments: COMPUTE
		for(i=0; i<numBeltPoints; i++) {
			//glVertex3f(mo1->bpntlist[i].p3.d[0], mo1->bpntlist[i].p3.d[1], mo1->bpntlist[i].p3.d[2]);
			p[0]=mo1->tpntlist[i].p3.d[0];
			p[1]=mo1->tpntlist[i].p3.d[1];
			p[2]=mo1->tpntlist[i].p3.d[2];

			quatToRotationMatrix(&(mo1->bquatlist[i]),mat);
			if(beltnum==2 || beltnum==3 || beltnum==4 || beltnum==5 || beltnum==10 || beltnum==11 || beltnum==12 || beltnum==13) {
				v[0] = p[0]; v[1] = p[1]; v[2] = p[2]-mo1->bwidth;
			}
			else  if(beltnum==0 || beltnum==1 || beltnum==6 || beltnum == 7 || beltnum==8 || beltnum==9){
				v[0] = p[0]-mo1->bwidth; v[1] = p[1]; v[2] = p[2];
			}
			mo1->br[i].p3.d[0] = v[0]*mat[0] + v[1]*mat[4] + v[2]*mat[8];
			mo1->br[i].p3.d[1] = v[0]*mat[1] + v[1]*mat[5] + v[2]*mat[9];
			mo1->br[i].p3.d[2] = scale*(v[0]*mat[2] + v[1]*mat[6] + v[2]*mat[10]);

			if(beltnum==2 || beltnum==3 || beltnum==4 || beltnum==5 || beltnum==10 || beltnum==11 || beltnum==12 || beltnum==13){
				v[0] = p[0]; v[1] = p[1]; v[2] = p[2]+mo1->bwidth;
			}
			else if(beltnum==0 || beltnum==1 || beltnum==6 || beltnum == 7 || beltnum==8 || beltnum==9){
				v[0] = p[0]+mo1->bwidth; v[1] = p[1]; v[2] = p[2];
			}
			mo1->bl[i].p3.d[0] = v[0]*mat[0] + v[1]*mat[4] + v[2]*mat[8];
			mo1->bl[i].p3.d[1] = v[0]*mat[1] + v[1]*mat[5] + v[2]*mat[9];
			mo1->bl[i].p3.d[2] = scale*(v[0]*mat[2] + v[1]*mat[6] + v[2]*mat[10]);
		}

		//Belt Segments: SEGMENT-FRAMES
		glPushAttrib(GL_CURRENT_BIT | GL_ENABLE_BIT | GL_LIGHTING_BIT | GL_LINE_BIT | GL_POINT_BIT );
			glEnable(GL_BLEND);
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			glEnable(GL_LINE_SMOOTH);

			glLineWidth(2.0);
			glDisable(GL_LIGHTING);
			for(i=0;i<numBeltPoints; i++) {
				if(i<numBeltPoints-1) {
					p1[0] = mo1->bpntlist[i].p3.d[0];
					p1[1] = mo1->bpntlist[i].p3.d[1];
					p1[2] = mo1->bpntlist[i].p3.d[2];

					p2[0] = mo1->bpntlist[i+1].p3.d[0];
					p2[1] = mo1->bpntlist[i+1].p3.d[1];
					p2[2] = mo1->bpntlist[i+1].p3.d[2];

					v1[0] = p2[0]-p1[0];
					v1[1] = p2[1]-p1[1];
					v1[2] = p2[2]-p1[2];
					len = length3D(v1);
					v1[0] = v1[0]/len;
					v1[1] = v1[1]/len;
					v1[2] = v1[2]/len;

					p1[0] = mo1->br[i].p3.d[0];
					p1[1] = mo1->br[i].p3.d[1];
					p1[2] = mo1->br[i].p3.d[2];

					p2[0] = mo1->bl[i].p3.d[0];
					p2[1] = mo1->bl[i].p3.d[1];
					p2[2] = mo1->bl[i].p3.d[2];

					v2[0] = p2[0]-p1[0];
					v2[1] = p2[1]-p1[1];
					v2[2] = p2[2]-p1[2];
					len = length3D(v2);
					v2[0] = v2[0]/len;
					v2[1] = v2[1]/len;
					v2[2] = v2[2]/len;
					
					p1[0] = mo1->bpntlist[i].p3.d[0];
					p1[1] = mo1->bpntlist[i].p3.d[1];
					p1[2] = mo1->bpntlist[i].p3.d[2];
					
					//x-axis
					if(mo1->showFrame && i%20==0) {
						glPushMatrix();
							glTranslatef(p1[0],p1[1],p1[2]);
							glScalef(frmscale,frmscale,frmscale);
							glColor3f(0.0f,0.9f,0.0f);
							glBegin(GL_LINES);
								glVertex3f(0.0f,0.0f,0.0f);
								glVertex3f(v2[0],v2[1],v2[2]);
							glEnd();
						glPopMatrix();
						if(i==0) {
							initFrame[0] = v2[0];
							initFrame[1] = v2[1];
							initFrame[2] = v2[2];
						}
					}
				
					cross3D(v1,v2,v3);
					len=length3D(v3);
					v3[0] = v3[0]/len;
					v3[1] = v3[1]/len;
					v3[2] = v3[2]/len;

					mo1->npntlist[i].p3.d[0] = v3[0];
					mo1->npntlist[i].p3.d[1] = v3[1];
					mo1->npntlist[i].p3.d[2] = v3[2];

					//y-axis
					if(mo1->showFrame && i%20==0) {
						glPushMatrix();
							glTranslatef(p1[0],p1[1],p1[2]);
							glScalef(frmscale,frmscale,frmscale);
							glColor3f(0.0f,0.0f,0.9f);
							glBegin(GL_LINES);
								glVertex3f(0.0f,0.0f,0.0f);
								glVertex3f(v3[0],v3[1],v3[2]);
							glEnd();
						glPopMatrix();
						if(i==0) {
							initFrame[3] = v3[0];
							initFrame[4] = v3[1];
							initFrame[5] = v3[2];
						}
					}

					cross3D(v2,v3,v1);
					len=length3D(v1);
					v1[0] = v1[0]/len;
					v1[1] = v1[1]/len;
					v1[2] = v1[2]/len;

					//z-axis
					if(mo1->showFrame && i%20==0) {
						glPushMatrix();
							glTranslatef(p1[0],p1[1],p1[2]);
							glScalef(frmscale,frmscale,frmscale);
							glColor3f(0.9f,0.0f,0.0f);
							glBegin(GL_LINES);
								glVertex3f(0.0f,0.0f,0.0f);
								glVertex3f(v1[0],v1[1],v1[2]);
							glEnd();
						glPopMatrix();
						if(i==0) {
	                        
							initFrame[6] = v1[0];
							initFrame[7] = v1[1];
							initFrame[8] = v1[2];
						}
					}
				}
				else if(i==numBeltPoints-1 && beltnum==0) {
                    p1[0] = mo1->bpntlist[numBeltPoints-1].p3.d[0];
                    p1[1] = mo1->bpntlist[numBeltPoints-1].p3.d[1];
                    p1[2] = mo1->bpntlist[numBeltPoints-1].p3.d[2];

                    quatToRotationMatrixf(&(mo1->bquatlist[i]),matf);
                    glPushMatrix();
                    glMultMatrixf(matf);
                    //x-axis
                    if(mo1->showFrame) {
                        glPushMatrix();
                        glTranslatef(p1[0],p1[1],p1[2]);
                        glScalef(frmscale,frmscale,frmscale);
                        glColor3f(0.0f,0.9f,0.0f);
                        glBegin(GL_LINES);
                        glVertex3f(0.0f,0.0f,0.0f);
                        glVertex3f(initFrame[0],initFrame[1],initFrame[2]);
                        glEnd();
                        glPopMatrix();
                    }
                        //y-axis
                    if(mo1->showFrame) {
                        glPushMatrix();
                        glTranslatef(p1[0],p1[1],p1[2]);
                        glScalef(frmscale,frmscale,frmscale);
                        glColor3f(0.0f,0.0f,0.9f);
                        glBegin(GL_LINES);
                        glVertex3f(0.0f,0.0f,0.0f);
                        glVertex3f(initFrame[3],initFrame[4],initFrame[5]);
                        glEnd();
                        glPopMatrix();
                    }
                        //z-axis
                    if(mo1->showFrame) {
                        glPushMatrix();
                        glTranslatef(p1[0],p1[1],p1[2]);
                        glScalef(frmscale,frmscale,frmscale);
                        glColor3f(0.9f,0.0f,0.0f);
                        glBegin(GL_LINES);
                        glVertex3f(0.0f,0.0f,0.0f);
                        glVertex3f(initFrame[6],initFrame[7],initFrame[8]);
                        glEnd();
                        glPopMatrix();
                    }
                    glPopMatrix();
                    
                }
			}
			mo1->npntlist[numBeltPoints-1].p3.d[0]=mo1->npntlist[numBeltPoints-2].p3.d[0];
			mo1->npntlist[numBeltPoints-1].p3.d[1]=mo1->npntlist[numBeltPoints-2].p3.d[1];
			mo1->npntlist[numBeltPoints-1].p3.d[2]=mo1->npntlist[numBeltPoints-2].p3.d[2];
		glPopAttrib();


		//Belt Segments: RENDER
		if(mo1->flag_track_belt != -1)
			mo1->showEdge = 1;
		if(mo1->flag_track_belt == -1) {
			if(mo1->showFace) {
			glPushAttrib(GL_CURRENT_BIT | GL_ENABLE_BIT | GL_LIGHTING_BIT | GL_LINE_BIT | GL_POINT_BIT );
			
				if(mo1->bpattern==BELT_PATTERN_SEGMENTS)
					glEnable(GL_COLOR_MATERIAL);
				else if(mo1->bpattern==BELT_PATTERN_DOUBLE_SIDED)
					glDisable(GL_COLOR_MATERIAL);

				glEnable(GL_NORMALIZE);
				glEnable(GL_LIGHTING);
				glDisable(GL_CULL_FACE);
				glLineWidth(5.0f);

				glEnable(GL_POLYGON_OFFSET_FILL);
				glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
				glPolygonOffset(0.0f,-2.0f);
				
				glMaterialfv( GL_FRONT, GL_AMBIENT, mat_ambient);
				glMaterialfv( GL_FRONT, GL_DIFFUSE, mat_diffuse);
				glMaterialfv( GL_FRONT, GL_SPECULAR, mat_specular);
				glMaterialf ( GL_FRONT, GL_SHININESS, 100.0f);
				glMaterialfv( GL_FRONT, GL_EMISSION, no_mat);
				glColorMaterial(GL_FRONT, GL_TRUE);
				
				glMaterialfv( GL_BACK, GL_AMBIENT, mat_ambient2);
				glMaterialfv( GL_BACK, GL_DIFFUSE, mat_diffuse2);
				glMaterialfv( GL_BACK, GL_SPECULAR, mat_specular2);
				glMaterialf ( GL_BACK, GL_SHININESS, 100.0f);
				glMaterialfv( GL_BACK, GL_EMISSION, no_mat2);
				glColorMaterial(GL_BACK, GL_TRUE);

				glBegin(GL_QUADS);
				for(i=1; i<numBeltPoints; i++) {
					if(i%30==0 || i%30==1 || i%30==2 || i%30==3 || i%30==4 || i%30==5 || i%30==6 || i%30==7|| i%30==8 || i%30==9 )
						glColor3f(0.5f,0.5f,0.1f);
					else
						glColor3f(color[0],color[1],color[2]);

					glNormal3f(mo1->npntlist[i].p3.d[0], mo1->npntlist[i].p3.d[1], mo1->npntlist[i].p3.d[2]);
					glVertex3f(mo1->br[i-1].p3.d[0], mo1->br[i-1].p3.d[1],  mo1->br[i-1].p3.d[2]);
					glVertex3f(mo1->br[i].p3.d[0],	 mo1->br[i].p3.d[1],	mo1->br[i].p3.d[2]);
					glVertex3f(mo1->bl[i].p3.d[0],   mo1->bl[i].p3.d[1],	mo1->bl[i].p3.d[2]);
					glVertex3f(mo1->bl[i-1].p3.d[0], mo1->bl[i-1].p3.d[1],  mo1->bl[i-1].p3.d[2]);
				}
				glEnd();
			
			glPopAttrib();
			}
		}
		else {
			if(mo1->showFace && beltnum==mo1->flag_track_belt) {
			glPushAttrib(GL_CURRENT_BIT | GL_ENABLE_BIT | GL_LIGHTING_BIT | GL_LINE_BIT | GL_POINT_BIT );
				
				if(mo1->bpattern==BELT_PATTERN_SEGMENTS)
					glEnable(GL_COLOR_MATERIAL);
				else if(mo1->bpattern==BELT_PATTERN_DOUBLE_SIDED)
					glDisable(GL_COLOR_MATERIAL);

				glEnable(GL_NORMALIZE);
				glEnable(GL_LIGHTING);
				glDisable(GL_CULL_FACE);
				glLineWidth(5.0f);

				glEnable(GL_POLYGON_OFFSET_FILL);
				glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
				glPolygonOffset(0.0f,-2.0f);
				
				glMaterialfv( GL_FRONT_AND_BACK, GL_AMBIENT, mat_ambient3);
				glMaterialfv( GL_FRONT_AND_BACK, GL_DIFFUSE, mat_diffuse3);
				glMaterialfv( GL_FRONT_AND_BACK, GL_SPECULAR, mat_specular3);
				glMaterialf ( GL_FRONT_AND_BACK, GL_SHININESS, 100.0f);
				glMaterialfv( GL_FRONT_AND_BACK, GL_EMISSION, no_mat3);
				glColorMaterial(GL_FRONT_AND_BACK, GL_TRUE);

				glMaterialfv( GL_BACK, GL_AMBIENT, mat_ambient4);
				glMaterialfv( GL_BACK, GL_DIFFUSE, mat_diffuse4);
				glMaterialfv( GL_BACK, GL_SPECULAR, mat_specular4);
				glMaterialf ( GL_BACK, GL_SHININESS, 100.0f);
				glMaterialfv( GL_BACK, GL_EMISSION, no_mat4);
				glColorMaterial(GL_BACK, GL_TRUE);

				glBegin(GL_QUADS);
				for(i=1; i<numBeltPoints; i++) {
					if(i%30==0 || i%30==1 || i%30==2 || i%30==3 || i%30==4 || i%30==5 || i%30==6 || i%30==7|| i%30==8 || i%30==9 )
						glColor3f(0.5f,0.5f,0.1f);
					else
						glColor3f(color[0],color[1],color[2]);

					glNormal3f(mo1->npntlist[i].p3.d[0], mo1->npntlist[i].p3.d[1], mo1->npntlist[i].p3.d[2]);
					glVertex3f(mo1->br[i-1].p3.d[0], mo1->br[i-1].p3.d[1],  mo1->br[i-1].p3.d[2]);
					glVertex3f(mo1->br[i].p3.d[0],	 mo1->br[i].p3.d[1],	mo1->br[i].p3.d[2]);
					glVertex3f(mo1->bl[i].p3.d[0],   mo1->bl[i].p3.d[1],	mo1->bl[i].p3.d[2]);
					glVertex3f(mo1->bl[i-1].p3.d[0], mo1->bl[i-1].p3.d[1],  mo1->bl[i-1].p3.d[2]);
				}
				glEnd();
			
			glPopAttrib();
			}
			
		}

				//Belt: Bounding Spheres
		if(beltnum==0) {
			if(mo1->inp_chkbox_belt_spheres_wire) {
				sphererad = 2.0/((double)mo1->bspheresFactor);
				dd = 0.0;
				i=0;
				
				glPushAttrib(GL_CURRENT_BIT | GL_LIGHTING_BIT | GL_LINE_WIDTH);

                glEnable(GL_POLYGON_OFFSET_FILL);
                glPolygonOffset(3.0,1.0);

                glEnable(GL_DEPTH_TEST);
				glEnable(GL_BLEND);
				glDisable(GL_LIGHTING);

                double scalefactor = 1.0;

				for(spherenum=0; spherenum<=mo1->bspheresFactor; spherenum++) {

					i = spherenum*(200/(mo1->bspheresFactor));
						
                        
						a = 1.0-(0.45*dd*scalefactor);
                        //printf("[%d] dd  = %f. a = %f\n",spherenum,dd,a);
					    glEnable(GL_CULL_FACE);
						//glCullFace(GL_FRONT);

						glLineWidth(0.3f);
						quatToRotationMatrixf(&(mo1->bquatlist[i-1]),matf);
						glPushMatrix();
							glMultMatrixf(matf);
                            //glColor4f(0.7f,0.7f,0.7f,a);
                            //glutSolidSphere(dd,40,40);
                            glColor4f(0.25f,0.25f,0.25f,a);
                            //glPolygonOffset(2.0, 1.0);
							glutWireSphere(dd,15,15);
						glPopMatrix();
	                    
                        glCullFace(GL_BACK);

					dd = dd+sphererad;
				}
                glDisable(GL_CULL_FACE);
                glDisable(GL_POLYGON_OFFSET_FILL);
				glPopAttrib();
			}

		}
		//Belt Segments: FRAME TRACKER
		if( beltnum==0 && mo1->showTracker) {
			trackerpos = mo1->btrackerpos;
			frmscale = 0.3;
			glPushAttrib( GL_ALL_ATTRIB_BITS );
				glEnable(GL_BLEND);
				glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
				glEnable(GL_LINE_SMOOTH);

				glEnable(GL_NORMALIZE);
				glEnable(GL_LIGHTING);
				glEnable(GL_COLOR_MATERIAL);
				glDisable(GL_CULL_FACE);
				glColor3f(0.6f,0.4f,0.1f);
					glPushMatrix();
						glTranslatef(mo1->bpntlist[trackerpos].p3.d[0], mo1->bpntlist[trackerpos].p3.d[1], mo1->bpntlist[trackerpos].p3.d[2]);
						//glutSolidCube(0.065);
                        glutSolidSphere(0.05,16,16);
					glPopMatrix();
				glPushAttrib( GL_ALL_ATTRIB_BITS );
					glLineWidth(4.0);
					glDisable(GL_LIGHTING);
					i = trackerpos;
					p1[0] = mo1->bpntlist[i].p3.d[0]; 
					p1[1] = mo1->bpntlist[i].p3.d[1];
					p1[2] = mo1->bpntlist[i].p3.d[2];

					p2[0] = mo1->bpntlist[i+1].p3.d[0];
					p2[1] = mo1->bpntlist[i+1].p3.d[1];
					p2[2] = mo1->bpntlist[i+1].p3.d[2];

					v1[0] = p2[0]-p1[0];
					v1[1] = p2[1]-p1[1];
					v1[2] = p2[2]-p1[2];
					len = length3D(v1);
					v1[0] = v1[0]/len;
					v1[1] = v1[1]/len;
					v1[2] = v1[2]/len;

					p1[0] = mo1->br[i].p3.d[0];
					p1[1] = mo1->br[i].p3.d[1];
					p1[2] = mo1->br[i].p3.d[2];

					p2[0] = mo1->bl[i].p3.d[0];
					p2[1] = mo1->bl[i].p3.d[1];
					p2[2] = mo1->bl[i].p3.d[2];

					v2[0] = p2[0]-p1[0];
					v2[1] = p2[1]-p1[1];
					v2[2] = p2[2]-p1[2];
					len = length3D(v2);
					v2[0] = v2[0]/len;
					v2[1] = v2[1]/len;
					v2[2] = v2[2]/len;
					
					p1[0] = mo1->bpntlist[i].p3.d[0];
					p1[1] = mo1->bpntlist[i].p3.d[1];
					p1[2] = mo1->bpntlist[i].p3.d[2];
					
				
					cross3D(v1,v2,v3);
					len=length3D(v3);
					v3[0] = v3[0]/len;
					v3[1] = v3[1]/len;
					v3[2] = v3[2]/len;


					cross3D(v2,v3,v1);
					len=length3D(v1);
					v1[0] = v1[0]/len;
					v1[1] = v1[1]/len;
					v1[2] = v1[2]/len;

					if(trackerpos<(numBeltPoints-1)){

						//x-axis
						glPushMatrix();
							glTranslatef(p1[0],p1[1],p1[2]);
							glScalef(frmscale,frmscale,frmscale);
							glColor3f(0.0f,0.75f,0.0f);
							glBegin(GL_LINES);
								glVertex3f(0.0f,0.0f,0.0f);
								glVertex3f(v2[0],v2[1],v2[2]);
							glEnd();
						glPopMatrix();

						//y-axis
						glPushMatrix();
							glTranslatef(p1[0],p1[1],p1[2]);
							glScalef(frmscale,frmscale,frmscale);
							glColor3f(0.0f,0.0f,0.75f);
							glBegin(GL_LINES);
								glVertex3f(0.0f,0.0f,0.0f);
								glVertex3f(v3[0],v3[1],v3[2]);
							glEnd();
						glPopMatrix();

						//z-axis
						glPushMatrix();
							glTranslatef(p1[0],p1[1],p1[2]);
							glScalef(frmscale,frmscale,frmscale);
							glColor3f(0.75f,0.0f,0.0f);
							glBegin(GL_LINES);
								glVertex3f(0.0f,0.0f,0.0f);
								glVertex3f(v1[0],v1[1],v1[2]);
							glEnd();
						glPopMatrix();

					}
					else if(i==(numBeltPoints-1)) {
						p1[0] = mo1->bpntlist[trackerpos].p3.d[0];
						p1[1] = mo1->bpntlist[trackerpos].p3.d[1];
						p1[2] = mo1->bpntlist[trackerpos].p3.d[2];
						quatToRotationMatrixf(&(mo1->bquatlist[i]),matf);
						glPushMatrix();
						glMultMatrixf(matf);
							//x-axis
							glPushMatrix();
							glTranslatef(p1[0],p1[1],p1[2]);
							glScalef(frmscale,frmscale,frmscale);
							glColor3f(0.0f,0.6f,0.0f);
							glBegin(GL_LINES);
							glVertex3f(0.0f,0.0f,0.0f);
							glVertex3f(1.0f,0.0f,0.0f);
							glEnd();
							glPopMatrix();
		                    
							//y-axis
							glPushMatrix();
							glTranslatef(p1[0],p1[1],p1[2]);
							glScalef(frmscale,frmscale,frmscale);
							glColor3f(0.0f,0.0f,0.6f);
							glBegin(GL_LINES);
							glVertex3f(0.0f,0.0f,0.0f);
							glVertex3f(0.0f,1.0f,0.0f);
							glEnd();
							glPopMatrix();

							//z-axis
							glPushMatrix();
							glTranslatef(p1[0],p1[1],p1[2]);
							glScalef(frmscale,frmscale,frmscale);
							glColor3f(0.6f,0.0f,0.0f);
							glBegin(GL_LINES);
							glVertex3f(0.0f,0.0f,0.0f);
							glVertex3f(0.0f,0.0f,1.0f);
							glEnd();
							glPopMatrix();

						glPopMatrix();
	                    
					}
					glPopAttrib();
				glPopAttrib();
			glPopAttrib();
		
		}



		//Belt Segments: EDGES
		if(mo1->showEdge) {
			glPushAttrib( GL_ALL_ATTRIB_BITS );

				glEnable(GL_LIGHTING);
				glDisable(GL_CULL_FACE);
				
				if(mo1->flag_blend)
					glEnable(GL_BLEND);
				else
					glDisable(GL_BLEND);
				glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
				glEnable(GL_LINE_SMOOTH);

				glLineWidth(0.5f);
	            
				if(mo1->bmultiple<=-1) {
	            
					glEnable(GL_COLOR_MATERIAL);                        
					//Right side edge
					glColor3f(0.0f,0.0f,0.5f);
					glMaterialfv( GL_FRONT, GL_AMBIENT, mat_ambient);
					glMaterialfv( GL_FRONT, GL_DIFFUSE, mat_diffuse);
					glMaterialfv( GL_FRONT, GL_SPECULAR, mat_specular);
					glMaterialf ( GL_FRONT, GL_SHININESS, 450.0f);
					glMaterialfv( GL_FRONT, GL_EMISSION, no_mat);
					glColorMaterial(GL_FRONT, GL_TRUE);

					renderThickenedCurve(numBeltPoints+1, 0.01, mo1->br);

					edges[0].p3.d[0] = mo1->br[0].p3.d[0];
					edges[0].p3.d[1] = mo1->br[0].p3.d[1];
					edges[0].p3.d[2] = mo1->br[0].p3.d[2];
	                
					edges[1].p3.d[0] = mo1->bl[0].p3.d[0];
					edges[1].p3.d[1] = mo1->bl[0].p3.d[1];
					edges[1].p3.d[2] = mo1->bl[0].p3.d[2];

					renderThickenedCurve(3,0.01, edges);

					//Left side edge
					glColor3f(0.5f,0.0f,0.2f);
					glMaterialfv( GL_FRONT, GL_AMBIENT, mat_ambient2);
					glMaterialfv( GL_FRONT, GL_DIFFUSE, mat_diffuse2);
					glMaterialfv( GL_FRONT, GL_SPECULAR, mat_specular2);
					glMaterialf ( GL_FRONT, GL_SHININESS, 450.0f);
					glMaterialfv( GL_FRONT, GL_EMISSION, no_mat);
					glColorMaterial(GL_FRONT, GL_TRUE);

					renderThickenedCurve(numBeltPoints+1, 0.01, mo1->bl);

					edges[0].p3.d[0] = mo1->br[numBeltPoints-1].p3.d[0];
					edges[0].p3.d[1] = mo1->br[numBeltPoints-1].p3.d[1];
					edges[0].p3.d[2] = mo1->br[numBeltPoints-1].p3.d[2];
	                
					edges[1].p3.d[0] = mo1->bl[numBeltPoints-1].p3.d[0];
					edges[1].p3.d[1] = mo1->bl[numBeltPoints-1].p3.d[1];
					edges[1].p3.d[2] = mo1->bl[numBeltPoints-1].p3.d[2];

					renderThickenedCurve(3,0.01, edges);
				}
				else {
					glDisable(GL_COLOR_MATERIAL);
					glDisable(GL_LIGHTING);

					for(i=1; i<numBeltPoints; i++) {

						if( (mo1->bhiderest==1) && (mo1->flag_track_belt>=0) ) {
							
							//display only that belt
							if( (beltnum==mo1->flag_track_belt) ) {
								glBegin(GL_LINES);
									glColor3f(0.2f,0.2f,0.2f);
									glVertex3f(mo1->br[i-1].p3.d[0]-displacement_edge,	mo1->br[i-1].p3.d[1],	mo1->br[i-1].p3.d[2]);
									glVertex3f(mo1->br[i].p3.d[0]-displacement_edge,	mo1->br[i].p3.d[1],		mo1->br[i].p3.d[2]);
								glEnd();
								glBegin(GL_LINES);
									glColor3f(0.2f,0.0f,0.2f);
									glVertex3f(mo1->bl[i-1].p3.d[0]+displacement_edge, mo1->bl[i-1].p3.d[1],	mo1->bl[i-1].p3.d[2]);
									glVertex3f(mo1->bl[i].p3.d[0]+displacement_edge,   mo1->bl[i].p3.d[1],		mo1->bl[i].p3.d[2]);
								glEnd();
								if(i%20==0) {
									glBegin(GL_LINES);
										glColor3f(0.65f,0.65f,0.65f);
										glVertex3f(mo1->br[i].p3.d[0]+displacement_edge, mo1->br[i].p3.d[1], mo1->br[i].p3.d[2]);
										glVertex3f(mo1->bl[i].p3.d[0]+displacement_edge, mo1->bl[i].p3.d[1], mo1->bl[i].p3.d[2]);
									glEnd();
								}
								if(i==(numBeltPoints-1)) {
									glBegin(GL_LINES);
										glVertex3f(mo1->br[i].p3.d[0], mo1->br[i].p3.d[1],  mo1->br[i].p3.d[2]);
										glVertex3f(mo1->bl[i].p3.d[0], mo1->bl[i].p3.d[1],	mo1->bl[i].p3.d[2]);
									glEnd();
								}
							}
						}
						else if( (mo1->bhiderest==1) && (mo1->flag_track_belt<0) ) {
							glBegin(GL_LINES);
								glColor3f(0.2f,0.2f,0.2f);
								glVertex3f(mo1->br[i-1].p3.d[0]-displacement_edge,	mo1->br[i-1].p3.d[1],	mo1->br[i-1].p3.d[2]);
								glVertex3f(mo1->br[i].p3.d[0]-displacement_edge,	mo1->br[i].p3.d[1],		mo1->br[i].p3.d[2]);
							glEnd();
							glBegin(GL_LINES);
								glColor3f(0.2f,0.0f,0.2f);
								glVertex3f(mo1->bl[i-1].p3.d[0]+displacement_edge, mo1->bl[i-1].p3.d[1],	mo1->bl[i-1].p3.d[2]);
								glVertex3f(mo1->bl[i].p3.d[0]+displacement_edge,   mo1->bl[i].p3.d[1],		mo1->bl[i].p3.d[2]);
							glEnd();
							if(i%20==0) {
								glBegin(GL_LINES);
									glColor3f(0.65f,0.65f,0.65f);
									glVertex3f(mo1->br[i].p3.d[0]+displacement_edge, mo1->br[i].p3.d[1], mo1->br[i].p3.d[2]);
									glVertex3f(mo1->bl[i].p3.d[0]+displacement_edge, mo1->bl[i].p3.d[1], mo1->bl[i].p3.d[2]);
								glEnd();
							}
							if(i==(numBeltPoints-1)) {
								glBegin(GL_LINES);
									glVertex3f(mo1->br[i].p3.d[0], mo1->br[i].p3.d[1],  mo1->br[i].p3.d[2]);
									glVertex3f(mo1->bl[i].p3.d[0], mo1->bl[i].p3.d[1],	mo1->bl[i].p3.d[2]);
								glEnd();
							}
						}
						else if(mo1->bhiderest==0) {
							glBegin(GL_LINES);
								glColor3f(0.2f,0.2f,0.2f);
								glVertex3f(mo1->br[i-1].p3.d[0]-displacement_edge,	mo1->br[i-1].p3.d[1],	mo1->br[i-1].p3.d[2]);
								glVertex3f(mo1->br[i].p3.d[0]-displacement_edge,	mo1->br[i].p3.d[1],		mo1->br[i].p3.d[2]);
							glEnd();
							glBegin(GL_LINES);
								glColor3f(0.2f,0.0f,0.2f);
								glVertex3f(mo1->bl[i-1].p3.d[0]+displacement_edge, mo1->bl[i-1].p3.d[1],	mo1->bl[i-1].p3.d[2]);
								glVertex3f(mo1->bl[i].p3.d[0]+displacement_edge,   mo1->bl[i].p3.d[1],		mo1->bl[i].p3.d[2]);
							glEnd();
							if(i%20==0) {
								glBegin(GL_LINES);
									glColor3f(0.65f,0.65f,0.65f);
									glVertex3f(mo1->br[i].p3.d[0]+displacement_edge, mo1->br[i].p3.d[1], mo1->br[i].p3.d[2]);
									glVertex3f(mo1->bl[i].p3.d[0]+displacement_edge, mo1->bl[i].p3.d[1], mo1->bl[i].p3.d[2]);
								glEnd();
							}
							if(i==(numBeltPoints-1)) {
								glBegin(GL_LINES);
									glVertex3f(mo1->br[i].p3.d[0], mo1->br[i].p3.d[1],  mo1->br[i].p3.d[2]);
									glVertex3f(mo1->bl[i].p3.d[0], mo1->bl[i].p3.d[1],	mo1->bl[i].p3.d[2]);
								glEnd();
							}
							
						}
					}
					glBegin(GL_LINES);
					glVertex3f(mo1->br[0].p3.d[0], mo1->br[0].p3.d[1],  mo1->br[0].p3.d[2]);
					glVertex3f(mo1->bl[0].p3.d[0], mo1->bl[0].p3.d[1],  mo1->bl[0].p3.d[2]);
					glEnd();
				}
				glPopAttrib();
		}
		
		if(mo1->inp_chkbox_belt_show_numbers) {
			glPushMatrix();
			glTranslatef(mo1->bpntlist[numBeltPoints-1].p3.d[0],
				mo1->bpntlist[numBeltPoints-1].p3.d[1],
				mo1->bpntlist[numBeltPoints-1].p3.d[2]);

				gluOrtho2D(-0.1,0.1,-0.1,0.1);
				//glMatrixMode(GL_MODELVIEW);
				glDisable(GL_LIGHTING);
				//if( (mo1->bhiderest==1) && (beltnum == mo1->flag_track_belt) && (mo1->flag_track_belt>=0) )
				//	glColor3f(0.2f,0.2f,0.2f);
				//else if( (mo1->bhiderest==1) && (beltnum != mo1->flag_track_belt) && (mo1->flag_track_belt>=0) )
				//	glColor3f(0.6f,0.6f,0.2f);
				//else if(mo1->bhiderest==0)
				//	glColor3f(0.2f,0.2f,0.2f);
				glColor3f(0.2f,0.2f,0.2f);
				if( (mo1->bhiderest==1) && (beltnum == mo1->flag_track_belt) && (mo1->flag_track_belt>=0) ) {
					sprintf(obj_qstr,"%d",beltnum+1);
					glRasterPos2f(0.0,0.0);
					text(obj_qstr);
				}
				else if( (mo1->bhiderest==1) && (beltnum != mo1->flag_track_belt) && (mo1->flag_track_belt>=0) ) {
				}
				else {
					sprintf(obj_qstr,"%d",beltnum+1);
					glRasterPos2f(0.0,0.0);
					text(obj_qstr);
				}
									
			glPopMatrix();
		}

	}

    //Belt Segments: CENTRAL-SPHERE
	if(mo1->showSphere) {
		glPushAttrib(GL_CURRENT_BIT | GL_ENABLE_BIT | GL_LIGHTING_BIT | GL_LINE_BIT | GL_POINT_BIT );
		
			glEnable(GL_COLOR_MATERIAL);

			glEnable(GL_NORMALIZE);
			glEnable(GL_LIGHTING);
			glDisable(GL_CULL_FACE);
			glLineWidth(1.5f);

			glMaterialfv( GL_FRONT, GL_AMBIENT, mat_ambient);
			glMaterialfv( GL_FRONT, GL_DIFFUSE, mat_diffuse);
			glMaterialfv( GL_FRONT, GL_SPECULAR, mat_specular);
			glMaterialf ( GL_FRONT, GL_SHININESS, 100.0f);
			glMaterialfv( GL_FRONT, GL_EMISSION, no_mat);
			glColorMaterial(GL_FRONT, GL_TRUE);

			glPushMatrix();
			glColor3f(0.3f,0.3f,0.3f);
			glTranslatef(0.0f,0.0f,0.0f);
			glutSolidSphere(0.2,32,32);
			glPopMatrix();
		glPopAttrib();
	}


}

/********************************************************************************
* Date   : Wed Jul 20 13:13:56 EST 2005
* Method : void renderThickenedCurve( int numpoints, double radius, Vertex *vec )
* Args   : number of points along belt, radius - thickness of tube, array of points
* Return : none
* Purpose: compute and render tubes along belt edges
********************************************************************************/
void renderThickenedCurve( int numpoints, double radius, Vertex *vec ) {

	double			p1[3];					//point 1
	double			p2[3];					//point 2
	double			v[3];					//vector p2-p1
	double			r[3];					//vector
	double			theta1	= 0.0;
	double			theta2	= 0.0;
	double			length	= 0.0;

	for(int i=0;i<(numpoints-2);i++) {

		p1[0] = vec[i].p3.d[0];
		p1[1] = vec[i].p3.d[1];
		p1[2] = vec[i].p3.d[2];
		
		p2[0] = vec[i+1].p3.d[0];
		p2[1] = vec[i+1].p3.d[1];
		p2[2] = vec[i+1].p3.d[2];

		minus3D(p2,p1,v);

		r[0] = -v[1]*v[2];
		r[1] =  v[0]*v[2];
		r[2] =  0.0;

		length = sqrt( v[0]*v[0] + v[1]*v[1] + v[2]*v[2] );
		double zero = 1.0e-3;

		if (fabs(v[2]) < zero) {
		    // rotation angle in x-y plane
            theta1 = (180/M_PI)*acos( v[0] / length );	
            if ( v[0] <= 0.0 ) theta1 = -theta1;
		}
		else {
		    // rotation angle
            theta2 = (180/M_PI)*acos( v[2] / length );	
            if ( v[2] <= 0.0 ) theta2 = -theta2;
		}

        glPushMatrix();
                glTranslatef(p1[0], p1[1], p1[2] ); 
                gluSphere (qobjSPH, radius, 16, 16);
        glPopMatrix();

        glPushMatrix();
            // translate to point 1
            glTranslatef(p1[0], p1[1], p1[2] );			
            if(fabs(v[2])<=zero) {
                glRotatef(90.0,  0.0, 1.0, 0.0);
                if((v[0]<0.0 && v[1]>0.0) || (v[0]>0.0 && v[1]<0.0))
                    glRotatef(theta1, 1.0, 0.0, 0.0);
                else if ((v[0]<0.0 && v[1]<0.0) || (v[0]>0.0 && v[1]>0.0))
                    glRotatef(theta1, -1.0, 0.0, 0.0);
            }
            else {
                // Rotate about rotation vector
                glRotatef(theta2, r[0], r[1], r[2]);	
            }
            gluQuadricTexture(qobjCYL, GL_TRUE);
            gluCylinder (qobjCYL, radius, radius, length+0.01, 16, 16);
        glPopMatrix();
	}

	p1[0] = vec[numpoints-2].p3.d[0];
    p1[1] = vec[numpoints-2].p3.d[1];
    p1[2] = vec[numpoints-2].p3.d[2];
    glPushMatrix();
        glTranslatef(p1[0], p1[1], p1[2] ); 
        gluSphere (qobjSPH, radius, 16, 16);
    glPopMatrix();
}
